/***********************************************************
 * $Id: fancylog.h 1306 2014-08-26 19:51:04Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas
**
** A versatile logging facility, which logs to stderr or syslog and
** emulates syslog under Win32.
**
*****************************************************************************/

#ifndef __FANCYLOG_H__
#define __FANCYLOG_H__ 1

#include <stdarg.h>
#ifdef WIN32
#define	LOG_EMERG	0
#define	LOG_ALERT	1
#define	LOG_CRIT	2
#define	LOG_ERR		3
#define	LOG_WARNING	4
#define	LOG_NOTICE	5
#define	LOG_INFO	6
#define	LOG_DEBUG	7
#else
#include <syslog.h>
#endif

#ifdef WIN32
# ifdef __GNUC__
#  ifdef BUILD_fancylog_DLL
#   define FANCYLOG_API __attribute__((dllexport))
#  else
#   define FANCYLOG_API __attribute__((dllimport))
#  endif
# else
#  ifdef BUILD_fancylog_DLL
#   define FANCYLOG_API __declspec(dllexport)
#  else
#   define FANCYLOG_API __declspec(dllimport)
#  endif
# endif
#endif

#ifndef FANCYLOG_API
#define FANCYLOG_API
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __GNUC__
# ifdef _MSC_VER
#  define __inline__ __inline
# else
#  define __inline__ inline
# endif
#endif

  /* Each application my have its own fancylog_service structure 
     for holding an own facility mask and active verbosity */
  typedef struct fancylog_facility_st fancylog_facility ;
  typedef struct fancylog_service_st fancylog_service;

  struct fancylog_facility_st
  {
    const char *ident; // ident of facility of null for termination of list.
    unsigned facility_mask; // the mask for this facility.
  };

  struct fancylog_service_st
  {
    const char *ident;
    unsigned facility_mask; // the active facilities, which are to be debugged.
    int max_priority; // the maximal priority to log for active facilities.

    const fancylog_facility *facilities; // null-terminated list of all available facilities.
  };

#ifdef WIN32
  // set the log file path, i.e. the directory where logfiles shall be. Has to be set before calling fancylog_init().
  FANCYLOG_API void fancylog_set_log_path(const char *path);
#endif

#define FANCY_LOG_METHOD_STDERR 0
#define FANCY_LOG_METHOD_SYSLOG 1
#define FANCY_LOG_METHOD_STDERR_PLAIN 2

  // parse for method cmdline args and filter out recognized options, and open the log via fancylog_open.
  FANCYLOG_API void fancylog_init(int def_method, const char *ident, int *argc, char **argv);

  // parse for logging cmdline args and filter out recognized options.
  FANCYLOG_API void fancylog_parse_arg (fancylog_service *svc, int *argc, char **argv);

  typedef void (*vfancylog_impl_t)(int priority, const char *format, va_list ap);

  // use the function backend() as final logging output backend
  // this is not thread safe and should be called during the initial application startup
  FANCYLOG_API void fancylog_init_backend(vfancylog_impl_t backend, const char *ident);

  // The function, which physically does the logging.
  FANCYLOG_API void vfancylog_impl(int priority, const char *format, va_list ap);

  static void fancylog (const fancylog_service *svc,
                        int facility,
                        int priority, const char *format, ...)
#ifdef __GNUC__
      __attribute__((format(printf,4,5)))
#endif
    ;
  static void vfancylog (const fancylog_service *svc,
                         int facility,
                         int priority, const char *fmt, va_list ap);

  __inline__ void vfancylog (const fancylog_service *svc,
                             int facility,
                             int priority, const char *fmt, va_list ap)
  {
    if (priority <= svc->max_priority || (facility & svc->facility_mask))
      {
        va_list aq;
        va_copy(aq,ap);
        vfancylog_impl(priority,fmt,aq);
        va_end(aq);
      }
  }

  __inline__ void fancylog (const fancylog_service *svc,
                            int facility,
                            int priority, const char *fmt, ...)
  {
    va_list ap;
    va_start(ap, fmt);
    vfancylog(svc,facility,priority,fmt,ap); 
    va_end(ap);
  }

  // close the log. be sure no to cut short any destructors which might to log something.
  FANCYLOG_API void fancylog_closelog();


  // "global" log call not distincting by logservice or facility
  // created for internal use
  // may be useful for debug logging before and during detection of loglevel
  static void fancylog_global(int priority, const char *format, ...)
#ifdef __GNUC__
      __attribute__((format(printf,2,3)))
#endif
    ;

  __inline__ void fancylog_global(int priority, const char *fmt, ...)
  {
  	va_list aq;
  	va_start(aq,fmt);
  	vfancylog_impl(priority,fmt,aq);
  	va_end(aq);
  }


#ifdef __cplusplus
} // end of extern "C"
#endif

#endif
